/*-
 *  Copyright 2020 Beuth Hochschule für Technik Berlin, Hochschule für Technik Stuttgart
 * 
 *  This file is part of CityDoctor2.
 *
 *  CityDoctor2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CityDoctor2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with CityDoctor2.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.hft.stuttgart.citydoctor2.utils;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.xmlobjects.gml.model.base.Reference;
import org.xmlobjects.gml.model.geometry.DirectPosition;

import de.hft.stuttgart.citydoctor2.check.CheckError;
import de.hft.stuttgart.citydoctor2.check.ErrorId;
import de.hft.stuttgart.citydoctor2.check.QualityAdeErrorVisitor;
import de.hft.stuttgart.citydoctor2.datastructure.CityObject;
import de.hft.stuttgart.citydoctor2.math.Vector3d;
import de.hft.stuttgart.quality.model.enums.ResultType;
import de.hft.stuttgart.quality.model.properties.ValidationResultProperty;
import de.hft.stuttgart.quality.model.types.CityObjectProperties;
import de.hft.stuttgart.quality.model.types.Edge;
import de.hft.stuttgart.quality.model.types.Validation;
import de.hft.stuttgart.quality.model.types.ValidationResult;

public class QualityADEUtils {

	private QualityADEUtils() {
	}

	public static Edge convertEdge(de.hft.stuttgart.citydoctor2.datastructure.Edge e) {
		DirectPosition from = convertVertex(e.getFrom());
		DirectPosition to = convertVertex(e.getTo());
		Edge result = new Edge();
		result.setFrom(from);
		result.setTo(to);
		return result;
	}

	public static DirectPosition convertVertex(Vector3d v) {
		DirectPosition result = new DirectPosition();
		result.getValue().add(v.getX());
		result.getValue().add(v.getY());
		result.getValue().add(v.getZ());
		return result;
	}

	public static void writeQualityAde(CityObject co, Validation val) {
		ValidationResult res = new ValidationResult();
		res.setValidationID(new Reference(val));
		if (co.isValidated()) {
			List<CheckError> errors = new ArrayList<>();
			co.collectContainedErrors(errors);
			if (errors.isEmpty()) {
				res.setResultType(ResultType.OK);
			} else {
				res.setResultType(ResultType.ERROR);
				Set<CheckError> errorSet = new HashSet<>(errors);
				for (CheckError e : errorSet) {
					QualityAdeErrorVisitor visitor = new QualityAdeErrorVisitor(res);
					e.accept(visitor);
				}
			}
		} else {
			res.setResultType(ResultType.NOT_CHECKED);
		}
		CityObjectProperties property = new CityObjectProperties();
		property.setValidationResult(new ValidationResultProperty(res));
		co.getGmlObject().addADEProperty(property);
	}

	public static de.hft.stuttgart.quality.model.enums.ErrorId mapErrorIdToAdeId(ErrorId key) {
        return switch (key.getIdString()) {
            case "GE_R_NOT_CLOSED" -> de.hft.stuttgart.quality.model.enums.ErrorId.GE_R_NOT_CLOSED;
            case "GE_R_TOO_FEW_POINTS" -> de.hft.stuttgart.quality.model.enums.ErrorId.GE_R_TOO_FEW_POINTS;
            case "GE_R_CONSECUTIVE_POINTS_SAME" ->
                    de.hft.stuttgart.quality.model.enums.ErrorId.GE_R_CONSECUTIVE_POINTS_SAME;
            case "GE_R_SELF_INTERSECTION" -> de.hft.stuttgart.quality.model.enums.ErrorId.GE_R_SELF_INTERSECTION;
            case "GE_P_NON_PLANAR_POLYGON_NORMALS_DEVIATION" ->
                    de.hft.stuttgart.quality.model.enums.ErrorId.GE_P_NON_PLANAR_POLYGON_NORMALS_DEVIATION;
            case "GE_P_NON_PLANAR_POLYGON_DISTANCE_PLANE" ->
                    de.hft.stuttgart.quality.model.enums.ErrorId.GE_P_NON_PLANAR_POLYGON_DISTANCE_PLANE;
            case "GE_P_INTERIOR_DISCONNECTED" ->
                    de.hft.stuttgart.quality.model.enums.ErrorId.GE_P_INTERIOR_DISCONNECTED;
            case "GE_P_INTERSECTING_RINGS" -> de.hft.stuttgart.quality.model.enums.ErrorId.GE_P_INTERSECTING_RINGS;
            case "GE_P_HOLE_OUTSIDE" -> de.hft.stuttgart.quality.model.enums.ErrorId.GE_P_HOLE_OUTSIDE;
            case "GE_P_ORIENTATION_RINGS_SAME" ->
                    de.hft.stuttgart.quality.model.enums.ErrorId.GE_P_ORIENTATION_RINGS_SAME;
            case "GE_P_INNER_RINGS_NESTED" -> de.hft.stuttgart.quality.model.enums.ErrorId.GE_P_INNER_RINGS_NESTED;
            case "GE_S_TOO_FEW_POLYGONS" -> de.hft.stuttgart.quality.model.enums.ErrorId.GE_S_TOO_FEW_POLYGONS;
            case "GE_S_NOT_CLOSED" -> de.hft.stuttgart.quality.model.enums.ErrorId.GE_S_NOT_CLOSED;
            case "GE_S_NON_MANIFOLD_EDGE" -> de.hft.stuttgart.quality.model.enums.ErrorId.GE_S_NON_MANIFOLD_EDGE;
            case "GE_S_POLYGON_WRONG_ORIENTATION" ->
                    de.hft.stuttgart.quality.model.enums.ErrorId.GE_S_POLYGON_WRONG_ORIENTATION;
            case "GE_S_ALL_POLYGONS_WRONG_ORIENTATION" ->
                    de.hft.stuttgart.quality.model.enums.ErrorId.GE_S_ALL_POLYGONS_WRONG_ORIENTATION;
            case "GE_S_NON_MANIFOLD_VERTEX" -> de.hft.stuttgart.quality.model.enums.ErrorId.GE_S_NON_MANIFOLD_VERTEX;
            case "GE_S_SELF_INTERSECTION" -> de.hft.stuttgart.quality.model.enums.ErrorId.GE_S_SELF_INTERSECTION;
            case "GE_S_MULTIPLE_CONNECTED_COMPONENTS" ->
                    de.hft.stuttgart.quality.model.enums.ErrorId.GE_S_MULTIPLE_CONNECTED_COMPONENTS;
            case "SE_ATTRIBUTE_WRONG_VALUE" -> de.hft.stuttgart.quality.model.enums.ErrorId.SE_ATTRIBUTE_WRONG_VALUE;
            case "SE_ATTRIBUTE_MISSING" -> de.hft.stuttgart.quality.model.enums.ErrorId.SE_ATTRIBUTE_MISSING;
            case "SE_ATTRIBUTE_INVALID" -> de.hft.stuttgart.quality.model.enums.ErrorId.SE_ATTRIBUTE_INVALID;
            default -> null;
        };
	}

}
